; *******************************************************************
; EDU10 Lesson 5 - Chasing LEDs
;
; This shows how to read the Analog-to-Digital converter and use the
; results to change the rotation rate of the LEDs.
; The pot on the EDU10 board varies the voltage 
; coming in on RA2.
;
; The Analog-to-Digital is referenced to the same Vdd as the device, which 
; is provided by the USB cable and nominally is 5V.  The Analog-to-Digital
; returns the ratio of the voltage on Pin RA2 to 5V.  The Analog-to-Digital
; has a resolution of 10 bits, with 1023 representing 5V and 0 representing 0V.
;
; *******************************************************************
; * For more details see Microchip's 44-pin Demo Board User's Guide *
; * Chapter 3. 44-Pin Demo Board Lessons                            *
; * http://ww1.microchip.com/downloads/en/DeviceDoc/41296B.pdf      *
; *******************************************************************

#include <p16F882.inc>
	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

     cblock     0x20
Delay1           ; Assign an address to label Delay1
Delay2
Display          ; define a variable to hold the diplay
     endc
          
     org 0
Start:
     bsf       STATUS,RP0     ; select Register Bank 1
     movlw     0xFF
     movwf     TRISA          ; Make PORTA all input
	 movlw     b'00001111'
     movwf     TRISC          ; Make PORTC pins RC7 - RC4 outputs  
     movlw     0x00           ; Left Justified, Vdd-Vss referenced
     movwf     ADCON1
     bsf       STATUS,RP1     ; select Register Bank 3
     movlw     0x07           ; we want PORTA pins RA0-RA2 Analog
     movwf     ANSEL
     bcf       STATUS,RP0     ; back to Register Bank 0
     bcf       STATUS,RP1
     
     movlw     b'01001001'
     movwf     ADCON0         ; configure Analog-to-Digital for Fosc/8, Channel 2 (RA2), 
							  ; and turn on the Analog-to-Digital module
     movlw     0x80
     movwf     Display
MainLoop:
     movf      Display,w           ; Copy the display to the LEDs
     movwf     PORTC
     nop                           ; wait 5uS for Analog-to-Digital amp to settle and capacitor to charge.
     nop                           ; wait 1uS
     nop                           ; wait 1uS
     nop                           ; wait 1uS
     nop                           ; wait 1uS
     bsf       ADCON0,GO_DONE      ; start conversion
     btfsc     ADCON0,GO_DONE      ; this bit will change to zero when the conversion is complete
     goto      $-1
     movf      ADRESH,w            ; Copy the display to the LEDs
     addlw     .1
     movwf     Delay2

A2DDelayLoop:
     decfsz    Delay1,f            ; Waste time.  
     goto      A2DDelayLoop        ; The Inner loop takes 3 instructions per loop * 256 loops = 768 instructions
     decfsz    Delay2,f            ; The outer loop takes and additional 3 instructions per lap * 256 loops
     goto      A2DDelayLoop        ; (768+3) * 256 = 197376 instructions / 1M instructions per second = 0.197 sec.
                                   ; call it two-tenths of a second.

     movlw     .26                 ; Delay another 20mS 
     movwf     Delay2
TenmSdelay:     
     decfsz    Delay1,f
     goto      TenmSdelay
     decfsz    Delay2,f
     goto      TenmSdelay
     
Rotate:
     bcf       STATUS, C           ; Ensure we don't rotate a 1 from the carry into bit 7.
     rrf       Display,f
     btfsc     Display,3           ; Did the bit rotate into the RC3?
     bsf       Display,7           ; yes, put it into bit 7.

     goto      MainLoop
     
     end
     
