; *******************************************************************
; EDU10 Lesson 7 - Reversible LEDs
;
; This shows how to read the Analog-to-Digital converter and rotate
; a bit on the 4 bit LED display.
; The pot on the EDU10 board varies the voltage 
; coming in on in pin RA2.
;
; The switch SW1 is used to reverse the direction of rotation
;
; The Analog-to-Digital is referenced to the same Vdd as the device, which 
; is provided by the USB cable and nominally is 5V.  The Analog-to-Digital
; returns the ratio of the voltage on Pin RA0 to 5V.  The Analog-to-Digital
; has a resolution of 10 bits, with 1023 representing 5V and
; 0 representing 0V.
; *******************************************************************
; * For more details see Microchip's 44-pin Demo Board User's Guide *
; * Chapter 3. 44-Pin Demo Board Lessons                            *
; * http://ww1.microchip.com/downloads/en/DeviceDoc/41296B.pdf      *
; *******************************************************************

#include <p16F882.inc>
	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

     cblock     0x20
Delay1         ; Assign an address to label Delay1
Delay2
Display        ; define a variable to hold the diplay
Direction
LookingFor
     endc

     
     org 0
Start:
     bsf       STATUS,RP0     ; select Register Bank 1
     movlw     0xFF
     movwf     TRISA          ; Make PORTA all input
	 movlw     b'00001111'
     movwf     TRISC          ; make IO Pins RC4-RC7 output
     movlw     0x00           ; Left Justified, Vdd-Vss referenced
     movwf     ADCON1
     bsf       STATUS,RP1     ; select Register Bank 3
     movlw     0x07           ; we want PORTA pins 0-2 Analog
     movwf     ANSEL
     bcf       STATUS,RP0     ; back to Register Bank 0
     bcf       STATUS,RP1     
     movlw     b'01001001'
     movwf     ADCON0         ; configure Analog-to-Digital for Fosc/8, Channel 2 (RA2), 
						      ; and turn on the Analog-to-Digital module
     movlw     0x80
     movwf     Display
     clrf      Direction
     clrf      LookingFor     ; Looking for a 0 on the button
MainLoop:
     movf      Display,w      ; Copy the display to the LEDs
     movwf     PORTC
     nop                      ; wait 5uS for A2D amp to settle and capacitor to charge.
     nop                      ; wait 1uS
     nop                      ; wait 1uS
     nop                      ; wait 1uS
     nop                      ; wait 1uS
     bsf       ADCON0,GO_DONE ; start conversion
     btfsc     ADCON0,GO_DONE ; this bit will change to zero when the conversion is complete
     goto      $-1
     movf      ADRESH,w       ; Copy the display to the LEDs
     movwf     Delay2

A2DDelayLoop:
     incfsz    Delay1,f       ; Waste time.  
     goto      A2DDelayLoop   ; The Inner loop takes 3 instructions per loop * 256 loopss = 768 instructions
     incfsz    Delay2,f       ; The outer loop takes and additional 3 instructions per lap * 256 loops
     goto      A2DDelayLoop   ; (768+3) * 256 = 197376 instructions / 1M instructions per second = 0.197 sec.
                              ; call it a two-tenths of a second.

     movlw     .13            ; Delay another 10mS plus whatever was above
     movwf     Delay2
TenmSdelay:
     decfsz    Delay1,f
     goto      TenmSdelay
     decfsz    Delay2,f
     goto      TenmSdelay

     btfsc     LookingFor,0
     goto      LookingFor1
LookingFor0:
     btfss     PORTA,4        ; is the switch pressed (1)
     goto      Rotate
     bsf       LookingFor,0   ; yes  Next we'll be looking for a 1
     movlw     0xFF           ; load the W register incase we need it
     xorwf     Direction,f    ; yes, flip the direction bit
     goto      Rotate

LookingFor1:
     btfss     PORTA,4        ; is the switch pressed (1)
     bcf       LookingFor,0
     
Rotate:
     bcf       STATUS,C       ; ensure the carry bit is clear
     btfss     Direction,0
     goto      RotateLeft
RotateRight:
     rrf       Display,f
     btfsc     STATUS,C       ; Did the bit rotate into the carry?
     bsf       Display,7      ; yes, put it into bit 7.

     goto      MainLoop
RotateLeft:
     rlf       Display,f
     btfsc     STATUS,C       ; did it rotate out of the display
     bsf       Display,0      ; yes, put it into bit 0
     goto      MainLoop
     
     end
     
