; *******************************************************************
; EDU10 Lesson 11 - Indirection Register
;
; This shows using the FSR to implement a Moving Average Filter.
; The pot value is read via the Analog-to-Digital converter, then the result is
; averaged with the last 7 readings.  The High order bits are shown
; on the 4 bit LED display. 
;
; *******************************************************************
; * For more details see Microchip's 44-pin Demo Board User's Guide *
; * Chapter 3. 44-Pin Demo Board Lessons                            *
; * http://ww1.microchip.com/downloads/en/DeviceDoc/41296B.pdf      *
; *******************************************************************

#include <p16F882.inc>
	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

     cblock 0x20  
Queue:8          ; 8 bytes to hold last 8 entries
Delay:2          ; counter to limit delay
Sum:2            ; sum of last 8 entries
QueuePointer   
temp
     endc
          
     org 0
Start:
     bsf       STATUS,RP0     ; select Register Bank 1
     movlw     0xFF
     movwf     TRISA          ; Make PORTA all input
	 movlw     b'00001111'
     movwf     TRISC          ; make IO Pins RC4-RC7 output
     movlw     0x00           ; Left Justified, Vdd-Vss referenced
     movwf     ADCON1

     movlw     B'10000111'    ; configure Prescaler on Timer0, max prescale (/256)
     movwf     OPTION_REG     ; configure

     bsf       STATUS,RP1     ; select Register Bank 3
     movlw     0x07           ; we want PORTA pins 0-2 Analog
     movwf     ANSEL
     bcf       STATUS,RP0     ; back to Register Bank 0
     bcf       STATUS,RP1
     
     movlw     b'01001001'
     movwf     ADCON0         ; configure Analog-to-Digital for Fosc/8, Channel 2 (RA2), 
						      ; and turn on the Analog-to-Digital module
     call      FilterInit     ; initialize the moving average filter
MainLoop:
     call      Delay200mS
     bsf       ADCON0,GO_DONE ; start Analog-to-Digital conversion
     btfsc     ADCON0,GO_DONE ; this bit will change to zero when the conversion is complete
     goto      $-1
     movf      ADRESH,w       ; read the Analog-to-Digital converter
     call      Filter         ; send it to the filter
     movwf     PORTC          ; display the filtered value
     goto      MainLoop

FilterInit:
     clrf      QueuePointer
     clrf      Queue
     clrf      Queue+1
     clrf      Queue+2
     clrf      Queue+3
     clrf      Queue+4
     clrf      Queue+5
     clrf      Queue+6
     clrf      Queue+7
     return
     
Filter:
     movwf     temp           ; save the latest value 
     MOVF      QueuePointer,w
     andlw     b'00000111'    ; mask it not to overflow
     addlw     Queue
     MOVWF     FSR            ; load FSR with the queue pointer
     MOVF      temp,w
     MOVWF     INDF           ; Write the new value to the queue
     INCF      QueuePointer, f ; Advance the pointer

     movlw     Queue
     movwf     FSR     
     clrf      Sum
     clrf      Sum+1
     movlw     0x08
     movwf     temp
Summing:
     movf      INDF,w                              
     addwf     Sum,f
     btfsc     STATUS,C       ; did it overflow
     incf      Sum+1
     decfsz    temp
     goto      Summing
                              
     bcf       STATUS,C       ; clear the carry
     rrf       Sum+1,f
     rrf       Sum,f          ; divide by 2 
     
     bcf       STATUS,C       ; clear the carry
     rrf       Sum+1,f
     rrf       Sum,f          ; divide by 4

     bcf       STATUS,C       ; clear the carry
     rrf       Sum+1,f
     rrf       Sum,w          ; divide by 8 and load Wreg with the answer

     return     
     
Delay200mS:
     decfsz    Delay,f        ; delay 
     goto      $-1
     decfsz    Delay+1,f      ; delay 768uS
     goto      $-3
     return
          
     end
