   
; *******************************************************************
; RESET:
; LDR is programmed as light detector. 
; LDR turns LED LD8 on/off depending on light.
; Potentiometer RV1 changes display brightness.
; If NTC voltage <1V or >4V display "E" (Error).
;
; SW1:
; Turn on LD7
; Display 7
;
; SW2:
; Turn on LD6
; Display 6
;
; SW3:
; Turn on LD5
; Display 5 
;
; SW4:
; Turn on LD4
; Beep
; Display all segments+dot
; *******************************************************************

#include <p16F882.inc>
	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

	cblock     0x20
tmp
tmp1
Delay1
Delay2
Delay3
intensity
	endc
     
	org 0x00
	goto	Start
	org		0x100
Start:
	bsf     STATUS,RP0       		; select Register Bank 1
	movlw   B'11110111'
	movwf   TRISA            		; Make PORTA input except pin RA3 as output
	clrf    TRISB              		; Make PORTB all output
	clrf    TRISC               	; Make PORTC all output
	bsf		STATUS,RP1     			; select Register Bank 3
	movlw	b'00000111'    			; we want 0-2 PORTA pins Analog
	movwf	ANSEL
	bcf		STATUS,RP0     			; back to Register Bank 0
	bcf		STATUS,RP1
	clrf	PORTA            	
	clrf	PORTB            		; PORTB LEDs all off
	clrf	PORTC            		; PORTC LEDs all off
	clrf	tmp
	clrf	ADCON1					; Analog-to-Digital Left justified, Voltage Reference VDD-VSS
	movlw	b'01000001'
	movwf	ADCON0         			; configure Analog-to-Digital for Fosc/8 and turn on the Analog-to-Digital module
	clrf	CCPR1L
	movlw 	b'00001100' 			; enable PWM mode
	movwf 	CCP1CON 
	movlw	b'00000100'				; timer2 on, 1:1 prescale
	movwf	T2CON

MainLoop:
	movf	PORTA,w	
	andlw	B'11110000'			; check switches
	btfss	STATUS,Z
	movwf	tmp

	movf	tmp,w	
	andlw	B'00010000'			; SW1 was pressed
	btfss	STATUS,Z
	goto	SW1_was_pressed
	movf	tmp,w	
	andlw	B'00100000'			; SW2 was pressed
	btfss	STATUS,Z
	goto	SW2_was_pressed
	movf	tmp,w	
	andlw	B'01000000'			; SW3 was pressed
	btfss	STATUS,Z
	goto	SW3_was_pressed
	movf	tmp,w	
	andlw	B'10000000'			; SW4 was pressed
	btfss	STATUS,Z
	goto	SW4_was_pressed

;								; Check NTC
	movlw	b'01000001'
	movwf	ADCON0         		; Analog-to-Digital Channel 0 (RA0), input from NTC
	call	Delay 				; Acquisition delay 8us
	bsf		ADCON0,GO_DONE 		; start conversion
	btfsc	ADCON0,GO_DONE 		; this bit will change to zero when the conversion is complete
	goto	$-1
	movf	ADRESH,w 
	sublw	d'204'				; check NTC error value Hi
	btfss	STATUS,C
	goto	show_e				; NTC Error
	movf	ADRESH,w  
	sublw	d'51'				; Check NTC error value Lo
	btfsc	STATUS,C
	goto	show_e				; NTC Error	

;								; Check LDR
	movlw	b'01000101'
	movwf	ADCON0         		; Analog-to-Digital Channel 1 (RA1), input from   LDR
	call	Delay 				; Acquisition delay 8us
	bsf		ADCON0,GO_DONE 		; start conversion
	btfsc	ADCON0,GO_DONE 		; this bit will change to zero when the conversion is complete
	goto	$-1
	movf	ADRESH,w
	andlw	B'10000000'
	movwf	tmp1
	bcf		STATUS,C
	rrf		tmp1,f 				; dim the LED
	rrf		tmp1,w     			; dim the LED
	movwf	CCPR1L				; set Duty Cycle bits 9 - 2

;								; Check potentiometer
	movlw	b'01001001'
	movwf	ADCON0         		; Analog-to-Digital Channel 2 (RA2), input from  RV1
	call	Delay 				; Acquisition delay 8us
	bsf		ADCON0,GO_DONE 		; start conversion
	btfsc	ADCON0,GO_DONE 		; this bit will change to zero when the conversion is complete
	goto	$-1
	movf	ADRESH,w 
	addlw	0x01	
	movwf	intensity    	
	movf	ADRESH,w
	sublw	0xff
	addlw	0x01	
	movwf	tmp1
	clrw						; indicate 0 while idle
	call	Display
Keep_display_on
	call	Delay
	decfsz	intensity
	goto    Keep_display_on	
	clrw
	movwf	PORTB				; display off
Keep_display_off
	call	Delay
	decfsz	tmp1
	goto    Keep_display_off

	goto    MainLoop	

SW1_was_pressed
	clrf	CCPR1L				; clear Duty Cycle bits 9 - 2
	movlw	B'10000000'			; LD7 on
	movwf	PORTC
	movlw	0x07
	call	Display
	goto	MainLoop

SW2_was_pressed
	clrf	CCPR1L				; clear Duty Cycle bits 9 - 2
	movlw	B'01000000'			; LD6 on
	movwf	PORTC
	movlw	0x06
	call	Display
	goto    MainLoop

SW3_was_pressed
	clrf	CCPR1L				; clear Duty Cycle bits 9 - 2
	movlw	B'00100000'			; LD5 on
	movwf	PORTC
	movlw	0x05
	call	Display
	goto    MainLoop

show_e							; NTC error indication
	movlw	d'20'
	call	Display
	goto    MainLoop	

SW4_was_pressed					; LD4 on and sound buzzer
	clrf	Delay1
	clrf	Delay2
	clrf	CCPR1L				; clear Duty Cycle bits 9 - 2
	movlw	d'18'
	call	Display
	movlw	d'16'				; set total tone length
	movwf	Delay3
	movlw	d'50'			
	movwf	tmp1				; set average tone pitch				
loop1
	decfsz	Delay1,f       		; Waste time.  
	goto	loop1    
	movf	PORTC,w	;read 
	xorlw	0x01				; invert buzzer bit
	andlw	0x01
	iorlw	B'00010000'			; LD4 on
	movwf	PORTC				; sounder to PortC
	movf	tmp1,w
	movwf	Delay1				
loop2
	decfsz	Delay1,f       		; Waste time.  
	goto	loop2 
	movf	tmp1,w
	movwf	Delay1				
	movf	Delay2,w
	andlw	b'0000000111'		; check if pitch will be changed
	btfss	STATUS,Z
	goto	no_pitch_change		; no pitch change
	btfsc	Delay3,0
	incf	tmp1				; dec pitch
	btfss	Delay3,0
	decf	tmp1				; inc pitch
no_pitch_change
	decfsz  Delay2
	goto	loop1			
	decfsz  Delay3
	goto	loop1
	clrf	tmp
	goto    MainLoop


Delay 							; Delay 8us
	nop
	nop
	nop 
	return	

	org	0x10
Display
	clrf	PCLATH
	call	SevenSegmentDecode
	movwf	PORTB		
	return

SevenSegmentDecode
	addwf	PCL,F	
	retlw	B'00111111' ;decode 0
	retlw	B'00000110' ;decode 1
	retlw	B'01011011' ;decode 2
	retlw	B'01001111' ;decode 3
	retlw	B'01100110' ;decode 4
	retlw	B'01101101' ;decode 5
	retlw	B'01111101' ;decode 6
	retlw	B'00000111' ;decode 7
	retlw	B'01111111' ;decode 8
	retlw	B'01101111' ;decode 9
	retlw	B'10111111' ;decode 0.
	retlw	B'10000110' ;decode 1.
	retlw	B'11011011' ;decode 2.
	retlw	B'11001111' ;decode 3.
	retlw	B'11100110' ;decode 4.
	retlw	B'11101101' ;decode 5.
	retlw	B'11111101' ;decode 6.
	retlw	B'10000111' ;decode 7.
	retlw	B'11111111' ;decode 8.
	retlw	B'11101111' ;decode 9.
	retlw	B'01111001' ;decode E

Continue


     end
