; *******************************************************************
; EDU10 Lesson 16 - Proximity Detector
; This code uses the PIC16F882 ADC to read the value of the VDR.
; The bright LED LD8 acts as light transmitter.
; If the reflection from surface of an external object is
; detected, a buzzer 'beep' is generated.
; The beep pitch is proportional to the reflection (distance of object).
; *******************************************************************


#include <p16F882.inc>
	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

	cblock     0x20 
tmp
delay1
delay2
tmp1
tmp2
	endc
     
	org 0x00
	goto	Start
Start:
	bsf     STATUS,RP0       		; select Register Bank 1
	movlw   B'11111111'
	movwf   TRISA            		; Make PORTA all input 
	clrf    TRISC               	; Make PORTC all output
	bsf		STATUS,RP1     			; select Register Bank 3
	movlw	b'00000111'    			; we want 0-2 PORTA pins Analog
	movwf	ANSEL
	bcf		STATUS,RP0     			; back to Register Bank 0
	bcf		STATUS,RP1
	clrf	PORTC            		; PORTC LEDs all off
	clrf	ADCON1					; Analog-to-Digital Left justified, Voltage Reference VDD-VSS
	movlw	b'01000001'
	movwf	ADCON0         			; configure Analog-to-Digital for Fosc/8 and turn on the Analog-to-Digital module
	movlw	b'01000101'
	movwf	ADCON0         		; Analog-to-Digital Channel 1 (RA1), input from   LDR
MainLoop:
	bsf		PORTC,2				; bright LED on
	call	Delay 				
	bsf		ADCON0,GO_DONE 		; start conversion
	btfsc	ADCON0,GO_DONE 		; this bit will change to zero when the conversion is complete
	goto	$-1
	movf	ADRESH,w
	movwf	tmp1				; A/D result when the light is on
	movwf	tmp

	bcf		PORTC,2				; bright LED off
	call	Delay 				
	bsf		ADCON0,GO_DONE 		; start conversion
	btfsc	ADCON0,GO_DONE 		; this bit will change to zero when the conversion is complete
	goto	$-1
	movf	ADRESH,w 
	movwf	tmp2				; A/D result when the light is off

	subwf	tmp1,w				; check if  A/D result with light on > A/D result with light off
	btfsc	STATUS,C
	goto	Detected
	goto    MainLoop
Detected
	movf	tmp2,w
	subwf	tmp1,f
	subwf	tmp,f				; the sound pitch will be proportional to the A/D result difference 
	bcf		STATUS,C
	rrf		tmp1,f				; divide the result by 2
	bcf		STATUS,C
	rrf		tmp1,f				; divide the result by 4 to reduce the sensitivity
	movlw	d'255'
	andwf	tmp1,w
	btfss	STATUS,Z			; check if the result is > 0
	goto	Beep
	goto    MainLoop
Beep
	clrf    delay1				; reset the beep duration counter
	bcf		STATUS,C
	rrf		tmp,f
	movlw	d'64'
	addwf	tmp,f				; to get proper sound pitch range set tmp > 64						
Loop	
	movf	tmp,w
	movwf	tmp1				; put the pitch value to the delay counter
PitchLoop
	decfsz	tmp1
	goto 	PitchLoop
	movf	PORTC,w				; read PORTC
	xorlw	0x01				; invert buzzer bit
	andlw	0x01
	movwf	PORTC				; sounder to PORTC	
	decfsz	delay1       		; waste time.
	goto	Loop
	goto    MainLoop	

Delay 						
	movlw	d'80'
	movwf	delay1
	movwf	delay2
Loop1
	decfsz  delay1
	goto	Loop1
	decfsz  delay2
	goto	Loop1
	return	

    end
