/*
 * File:   main.c
 * Author: VEL255
 *
 * Created on 15/11/2012
 *
 **********************************************************************
 * C Compiler Demo for the EDU10 Board.
 * Proximity Detector
 * This code uses the PIC16F882 ADC to read the value of the VDR.
 * The bright LED LD8 acts as light transmitter.
 * If the reflection from surface of an external object is
 * detected, a buzzer 'beep' is generated.
 * The beep pitch is proportional to the reflection (distance of object).
 *
**********************************************************************/

//#include <stdio.h>
//#include <stdlib.h>
//#include <stdint.h>
#include <xc.h>

//------------------------------------------------------------------------------
//PRAGMA's

#pragma config  CPD =       OFF
#pragma config  BOREN =     OFF
#pragma config  IESO =      OFF
#pragma config  DEBUG =     OFF
#pragma config  FOSC =      INTRC_NOCLKOUT
#pragma config  FCMEN =     OFF
#pragma config  MCLRE =     ON
#pragma config  WDTE =      OFF
#pragma config  CP =        OFF
#pragma config  LVP =       OFF
#pragma config  PWRTE =     OFF
#pragma config  BOR4V =     BOR21V
#pragma config  WRT =       OFF

//------------------------------------------------------------------------------
//DEFINES

#define LD7     PORTCbits.RC7
#define LD6     PORTCbits.RC6
#define LD5     PORTCbits.RC5
#define LD4     PORTCbits.RC4
#define LD8     PORTCbits.RC2
#define buzzer  PORTCbits.RC0

#define SW1     PORTAbits.RA4
#define SW2     PORTAbits.RA5
#define SW3     PORTAbits.RA6
#define SW4     PORTAbits.RA7

//------------------------------------------------------------------------------
//DECLARATIONS

void init (void);
void beep (unsigned char pitch);
unsigned char detect (void);

//------------------------------------------------------------------------------
//CODE

//------------------------------------------------------------------------------
void main (void)
{
    unsigned char tmp;
    
    init();
    while(1)
    {
        tmp = detect();
        if (tmp)
            beep(tmp);
    }
}
//------------------------------------------------------------------------------
unsigned char detect (void)
{
    unsigned char i;
    unsigned int counter;
    unsigned int lightOn;
    unsigned int lightOff;
    lightOn = 0;
    lightOff = 0;
    LD8 = 1;                        // bright LED on
    i = 0xff;
    while (i--)
    {
        counter = 30;
        while (counter--);          // delay
        {
            ADCON0bits.GO = 1;      // start conversion
            while (ADCON0bits.GO);  // wait until done
            lightOn += ADRESH;
        }
    }
    LD8 = 0;                          // bright LED off
    i = 0xff;
    while (i--)
    {
        counter = 30;
        while (counter--);          // delay
        {
            ADCON0bits.GO = 1;      // start conversion
            while (ADCON0bits.GO);  // wait until done
            lightOff += ADRESH;
        }
    }
    if (lightOn > lightOff)
    {
        lightOn -= lightOff;
        lightOn >>= 8;              // divide the result by 256 
                                    // to reduce the sensitivity
        return lightOn;
    }
    else
        return 0;
}


//------------------------------------------------------------------------------
void beep (unsigned char pitch)
{
    unsigned char i; 
    unsigned int p;

    i = 100;                    // beep duration
    while (i--)
    {
        buzzer ^= 1;            // invert buzzer bit
        p = pitch + 40;         // pitch
        while (p--);
    }
}

//------------------------------------------------------------------------------
void init(void)
{
    //OSC
    OSCCONbits.IRCF0 = 1;        // internal osc = 8Mhz
    OSCCONbits.IRCF1 = 1;
    OSCCONbits.IRCF2 = 1;
    OSCCONbits.OSTS = 1;        // osc is selected by FOSC (pragma)
    OSCCONbits.SCS = 0;         // clock source defined by FOSC (pragma)

    //PORTA
    PORTA = 0b00000000;         // clear PORTA
    ANSEL = 0b00000111;         // 0-2 PORTA pins Analog
    TRISA = 0b11111111;         // all pins are input

    //PORTB
    PORTB = 0b00000000;         // clear PORTB
    TRISB = 0b00000000;         // all pins are output (7 SEGMENT)
    WPUB = 0b00000000;          // weak pull-ups disabled
    IOCB = 0b00000000;          // interrupt on change disabled

    //PORTC
    PORTC = 0b00000000;         // clear PORTC
    TRISC = 0b00000000;         // all pins are output

    //INITS
    ADCON0 = 0b10000101;  	// Analog-to-Digital FOSC/32, Channel 1 (RA1),
                                //   input from   LDR
}



