/* 
 * File:   main.c
 * Author: VEL255
 *
 * Created on 28/11/2012
 **********************************************************************
 * C Compiler Demo for the EDU10 Board.
 *
 * Timer interrupt is used to flash the LED.
 * The flash interval is adjusted by the potentiometer RV1.
 * ********************************************************************
 */

//------------------------------------------------------------------------------
//#include <stdio.h>
//#include <stdlib.h>
#include <xc.h>
//------------------------------------------------------------------------------
//PRAGMA's

#pragma config  CPD =       OFF
#pragma config  BOREN =     OFF
#pragma config  IESO =      OFF
#pragma config  DEBUG =     OFF
#pragma config  FOSC =      INTRC_NOCLKOUT
#pragma config  FCMEN =     OFF
#pragma config  MCLRE =     ON
#pragma config  WDTE =      OFF
#pragma config  CP =        OFF
#pragma config  LVP =       OFF
#pragma config  PWRTE =     OFF
#pragma config  BOR4V =     BOR21V
#pragma config  WRT =       OFF

//------------------------------------------------------------------------------
//DEFINES

#define LD7     PORTCbits.RC7
#define LD6     PORTCbits.RC6
#define LD5     PORTCbits.RC5
#define LD4     PORTCbits.RC4

#define SW1     PORTAbits.RA4
#define SW2     PORTAbits.RA5
#define SW3     PORTAbits.RA6
#define SW4     PORTAbits.RA7


//------------------------------------------------------------------------------
//DECLARATIONS

void init (void);

unsigned char tmp;

//------------------------------------------------------------------------------
//CODE

//------------------------------------------------------------------------------
void main (void)
{

    init();
    LD6 = 1;
    LD7 = 0;
    while(1)
    {
        ADCON0bits.GO = 1;       // start conversion
        while (ADCON0bits.GO);   // wait until done
        ADRESH >>= 1;            // divide result by 2
        ADRESH += 10;            // add 10 to limit the max. flash rate
        if (tmp > ADRESH)
        {
            LD7 ^= 1;            // toggle LED
            LD6 ^= 1;            // toggle LED
            tmp = 0;
        }        
    }
}
//------------------------------------------------------------------------------
/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    tmp++;
    INTCONbits.T0IF = 0;            // clear interrupt flag
}

//------------------------------------------------------------------------------
void init(void)
{
    //OSC
    OSCCONbits.IRCF0 = 1;        // internal osc = 8Mhz
    OSCCONbits.IRCF1 = 1;
    OSCCONbits.IRCF2 = 1;
    OSCCONbits.OSTS = 1;        // osc is selected by FOSC (pragma)
    OSCCONbits.SCS = 0;         // clock source defined by FOSC (pragma)

    //PORTA
    PORTA = 0b00000000;         // clear PORTA
    ANSEL = 0b00000000;         // all pins are digital
    TRISA = 0b11111111;         // all pins are input

    //PORTB
    PORTB = 0b00000000;         // clear PORTB
    TRISB = 0b00000000;         // all pins are output (7 SEGMENT)
    WPUB = 0b00000000;          // weak pull-ups disabled
    IOCB = 0b00000000;          // interrupt on change disabled

    //PORTC
    PORTC = 0b00000000;         // clear PORTC
    TRISC = 0b00000000;         // all pins are output

    //INITS
                                // Timer 0
    OPTION_REGbits.T0CS = 0;    // Internal instruction cycle clock (FOSC/4)
    OPTION_REGbits.PSA = 0;     // Prescaler is assigned to the Timer0 module
    OPTION_REGbits.PS = 0b100;  // Prescaler 1:32
    INTCONbits.T0IE = 1;
    INTCONbits.T0IE = 1;        // Enables the Timer0 interrupt
    INTCONbits.PEIE = 1;        // Enables all unmasked peripheral interrupts
    INTCONbits.GIE = 1;         // Enables all unmasked interrupts
                                // ADC
    ADCON0 = 0b10001001;        // Analog-to-Digital FOSC/32, Channel 2 (RA2),
                                // input from  RV1

    tmp = 0;
}

