/* 
 * File:   main.c
 * Author: VEL255
 *
 * Created on 15/11/2012
 **********************************************************************
 * C Compiler Demo for the EDU10 Board.
 * Analog-to-Digital converter
 * This code uses the PIC16F882 ADC to read the value of the VDR, NTC and RV1.
 * The eight most significant bits of the ADC result is converted
 * to value 000-255 and displayed on the 7-segment display.
 * Buttons are used to select the analog input device:
 * SW1: LDR
 * SW2: NTC
 * SW3: Potentiometer
 * SW4: Potentiometer is used to adjust the buzzer tone pitch.
**********************************************************************/

//#include <stdio.h>
//#include <stdlib.h>
//#include <stdint.h>
#include <xc.h>

//------------------------------------------------------------------------------
//PRAGMA's

#pragma config  CPD =       OFF
#pragma config  BOREN =     OFF
#pragma config  IESO =      OFF
#pragma config  DEBUG =     OFF
#pragma config  FOSC =      INTRC_NOCLKOUT
#pragma config  FCMEN =     OFF
#pragma config  MCLRE =     ON
#pragma config  WDTE =      OFF
#pragma config  CP =        OFF
#pragma config  LVP =       OFF
#pragma config  PWRTE =     OFF
#pragma config  BOR4V =     BOR21V
#pragma config  WRT =       OFF

//------------------------------------------------------------------------------
//DEFINES

#define LD7     PORTCbits.RC7
#define LD6     PORTCbits.RC6
#define LD5     PORTCbits.RC5
#define LD4     PORTCbits.RC4
#define buzzer  PORTCbits.RC0
#define offLD   PORTC &= 0b00001111

#define SW1     PORTAbits.RA4
#define SW2     PORTAbits.RA5
#define SW3     PORTAbits.RA6
#define SW4     PORTAbits.RA7

//------------------------------------------------------------------------------
//DECLARATIONS

void init (void);
unsigned char buttons (void);
void show(unsigned char value);
void beep (unsigned char pitch);
unsigned char ntc (void);
unsigned char ldr (void);
unsigned char pot (void);
void delay (void);
unsigned char delayDisplay (void);

unsigned char button;

const unsigned char segments7[] = {
    0b00111111, // 0
    0b00000110, // 1
    0b01011011, // 2
    0b01001111, // 3
    0b01100110, // 4
    0b01101101, // 5
    0b01111101, // 6
    0b00000111, // 7
    0b01111111, // 8
    0b01101111  // 9
};

//------------------------------------------------------------------------------
//CODE

//------------------------------------------------------------------------------
void main (void)
{
    init();
    while(1)
    {
        buttons();
        switch  (button)
        {
            case 1: show(ldr()); break;
            case 2: show(ntc()); break;
            case 3: show(pot()); break;
            case 4: PORTB = 0;
                    beep(pot()); break;
        }
    }
}

//------------------------------------------------------------------------------
unsigned char buttons (void)
{
    if (SW1 == 1)
    {
        offLD;
        LD7 = 1;
        button = 1;
        return 1;
    }
    if (SW2 == 1)
    {
        offLD;
        LD6 = 1;
        button = 2;
        return 1;
    }
    if (SW3 == 1)
    {
        offLD;
        LD5 = 1;
        button = 3;
        return 1;
    }
    if (SW4 == 1)
    {
        offLD;
        LD4 = 1;
        button = 4;
        return 1;
    }
    return 0;
}

//------------------------------------------------------------------------------
void show(unsigned char value)
{
	unsigned char p, o, cnt;
	char msg[3];

	o = value/100;
	msg[0] = o;
	p = (value-100*o)/10;
	msg[1] = p;
	msg[2] = value-o*100-p*10;

        PORTB = segments7[msg[0]];
        if (delayDisplay())
            return;
        PORTB = 128;
        if (delayDisplay())
            return;
        PORTB = segments7[msg[1]];
        if (delayDisplay())
            return;
        PORTB = 128;
        if (delayDisplay())
            return;
        PORTB = segments7[msg[2]];
        if (delayDisplay())
            return;
        PORTB = 0;
        for (cnt=0;cnt<6;cnt++)
        {
            if (delayDisplay())
            return;
        }
}

//------------------------------------------------------------------------------
unsigned char ldr (void)
{
    ADCON0 = 0b10000101;    // Analog-to-Digital FOSC/32, Channel 1 (RA1),
                            //   input from   LDR
    delay();                // Acquisition delay
    ADCON0bits.GO = 1;      // start conversion
    while (ADCON0bits.GO);  // wait until done
    return ADRESH;
}

//------------------------------------------------------------------------------
unsigned char ntc (void)
{

    ADCON0 = 0b10000001;    // Analog-to-Digital FOSC/32, Channel 0 (RA0),
                            //   input from NTC
    delay();                // Acquisition delay
    ADCON0bits.GO = 1;      // start conversion
    while (ADCON0bits.GO);  // wait until done
    return ADRESH;
}

//------------------------------------------------------------------------------
unsigned char pot (void)
{
        ADCON0 = 0b10001001;     // Analog-to-Digital FOSC/32, Channel 2 (RA2),
                                 //   input from  RV1
        delay();                 // Acquisition delay
        ADCON0bits.GO = 1;       // start conversion
        while (ADCON0bits.GO);   // wait until done
        return ADRESH;
}

//------------------------------------------------------------------------------
void delay (void)
{
    unsigned char i = 1;
    while (i--);                    // delay about 10us @ osc 8MHz
}

//------------------------------------------------------------------------------
unsigned char delayDisplay (void)
{
    unsigned int t;

    t = 15000;
    while (t--)
       if (buttons())
            return 1;
    return 0;
}

//------------------------------------------------------------------------------
void beep (unsigned char pitch)
{
    unsigned int p;
    buzzer ^= 1;                // invert buzzer bit
    p = 255-pitch + 50;         // pitch
    while (p--);
}

//------------------------------------------------------------------------------
void init(void)
{
    //OSC
    OSCCONbits.IRCF0 = 1;       // internal osc = 8Mhz
    OSCCONbits.IRCF1 = 1;
    OSCCONbits.IRCF2 = 1;
    OSCCONbits.OSTS = 1;        // osc is selected by FOSC (pragma)
    OSCCONbits.SCS = 0;         // clock source defined by FOSC (pragma)

    //PORTA
    PORTA = 0b00000000;         // clear PORTA
    ANSEL = 0b00000111;         // 0-2 PORTA pins Analog
    TRISA = 0b11111111;         // all pins are input

    //PORTB
    PORTB = 0b00000000;         // clear PORTB
    TRISB = 0b00000000;         // all pins are output (7 SEGMENT)
    WPUB = 0b00000000;          // weak pull-ups disabled
    IOCB = 0b00000000;          // interrupt on change disabled

    //PORTC
    PORTC = 0b00000000;         // clear PORTC
    TRISC = 0b00000000;         // all pins are output
}


